goog.provide 'app.AppComponent'

goog.require 'App'
goog.require 'app.ui.Loader'

class app.AppComponent
  ###*
    @param {app.EditorStore} editorStore
    @param {app.DocumentStore} docStore
    @param {app.editor.Document} editor
    @param {app.viewer.Document} viewer
    @param {app.formatter.Document} formatter
    @param {app.references.Editor} references
    @param {app.Routes} routes
    @constructor
  ###
  constructor: (editorStore, docStore, editor, viewer, formatter, references, routes) ->

    Viewer = viewer.reactClass
    Formatter = formatter.reactClass
    Editor = editor.reactClass
    References = references.reactClass
    {button, p, h1, h2, h3, a, div, ul, li} = React.DOM

    @reactClass = React.createClass(
      displayName: 'AppComponent'

      render: ->
        unless @props.appState.editingDocumentId
          return @chooseDocument()

        if routes.active == routes.home
          return @chooseDocument()

        unless @props.appState.editingDocument?
          loadResult = docStore.loadDocument(@props.appState.editingDocumentId)
          loadResult.then(@onDocumentLoad, @onDocumentLoadFail)
          return app.ui.Loader({text: 'Document is loading.'})

        div(
          {}
          ul(
            {className: 'nav nav-tabs', role: 'tabList'}
            @linkToModule(@props.activeModule, 'edit')
            @linkToModule(@props.activeModule, 'view')
            @linkToModule(@props.activeModule, 'format')
            li(
              {className: 'close-editor'}
              button({className: 'btn btn-warning', onClick: @onAppClose}, 'close')
            )
          )
          @getModule()
        )

      onAppClose: ->
        editorStore.setAppState(
          {editingDocumentId: '', editingDocument: null, shouldReloadDocs: true}
        )

      onAllDocsLoaded: (res) ->
        editorStore.setAppState {shouldReloadDocs: false, allDocs: res}

      onDocumentDelete: ->
        editorStore.setAppState {shouldReloadDocs: true}
        console.warn 'document deleted' if app.debug

      onDocumentLoad: ->
        @setEditingDocument()

      onDocumentLoadFail: (err) ->
        console.warn err if app.debug
        editorStore.setAppState {editingDocumentId: ''}

      ###*
        Saves documents silently even if nothing changed.
      ###
      exampleChosen: ->
        routes.edit.redirect()
        docStore.createDocumentFromExample()
        @setEditingDocument()
        editorStore.forceDocumentSave()

      existingChosen: (id) ->
        routes.edit.redirect()
        editorStore.setAppState({editingDocumentId: id, editingDocument: null})

      ###*
        @param {app.document.Document} doc
      ###
      deleteDocument: (doc) ->
        result = confirm "Do you really want to delete '#{doc.title}'?"
        if result
          editorStore.deleteDocument(doc).then(@onDocumentDelete, app.reportError)


      chooseDocument: ->
        if !@props.appState.allDocs? or @props.appState.shouldReloadDocs
          docsResult = docStore.loadAllDocs()
          docsResult.then(@onAllDocsLoaded, app.reportError)

        citation = "JavaScript technologies overview
Mozilla Developer Network, (2014). JavaScript technologies overview. [online] Available at: https://developer.mozilla.org/en-US/docs/Web/JavaScript/JavaScript_technologies_overview [Accessed 26 Dec. 2014]."

        div(
          {className: 'page-header'}
          h1({}, 'Choose document to edit')
          ul(
            {className: 'choose-list'}
            li(
              {className: 'choose-item'}
              h2({}, 'Example')
              p({}, 'Edit example document to get familiar with application. Document source is:')
              p({}, citation)
              button(
                {onClick: @exampleChosen, className: 'btn btn-primary'}, 'start with JavaScript technologies overview'
              )
            )
            li(
              {className: 'choose-item'}
              h2({}, 'New')
              p({}, 'Create and save new document to your browser.')
              button(
                {onClick: @newChosen, className: 'btn btn-warning'}, 'create new'
              )
            )
            li(
              {className: 'choose-item'}
              h2({}, 'Existing')
              p({}, 'Edit previously created document.')
              ul(
                {className: 'existing-list'}
                @renderExistingDocuments()
              )
            )
          )
        )

      renderExistingDocuments: ->
        if @props.appState.allDocs && @props.appState.allDocs.length
          @props.appState.allDocs.map @renderDocumentIcon
        else if @props.appState.allDocs
          'None document found.'
        else
          app.ui.Loader()

      renderDocumentIcon: (doc) ->
        li(
          {key: doc._id}
          button(
            {
              className: 'btn btn-primary'
              onClick: @existingChosen.bind(@, doc._id)
            }
            "open #{doc.title}"
          )
          button(
            {
              title: 'remove document'
              className: 'btn btn-danger delete-doc'
              onClick: @deleteDocument.bind(@, doc)
            }
            'x'
          )
        )

      ###*
        @return {React.ReactComponent}
      ###
      getModule: ->
        blockId = routes.active.params?.blockId

        return switch routes.active
          when routes.edit
            Editor({shouldActivateBlockId: blockId, appState: @props.appState})
          when routes.view
            Viewer({viewedBlockId: blockId})
          when routes.format
            Formatter()
          when routes.cite
            References()
          else
            console.error 'No module for route: ', routes.active if app.debug
            return null

      ###*
        @param {string} active
        @param {string} moduleName
      ###
      linkToModule: (active, moduleName) ->
        li(
          {className: (if active.match moduleName then 'active' else '')}
          a({href: routes[moduleName].url({module: moduleName})}, moduleName)
        )

      ###*
        See exampleChosen
      ###
      newChosen: ->
        routes.edit.redirect()
        docStore.createNewDocument()
        @setEditingDocument()
        editorStore.forceDocumentSave()

      setEditingDocument: ->
        documentObject = docStore.document
        editorStore.setDocument documentObject
        editorStore.setAppState({
          editingDocument: documentObject
          editingDocumentId: documentObject._id
        })

    )
