goog.provide 'app.editor.Document'

goog.require 'app.editor.Section'
goog.require 'app.editor.Block'
goog.require 'app.editor.Image'
goog.require 'app.AbstractType'
goog.require 'app.BlockType'
goog.require 'app.ui.AppendButton'
goog.require 'app.ui.Loader'
goog.require 'app.document.StructuredBlock'
goog.require 'app.document.Document'

class app.editor.Document
  ###*
    @param {app.DocumentStore} documentStore
    @param {app.EditorStore} editorStore
    @param {app.editor.Block} blockModule
    @param {app.editor.Section} sectionModule
    @param {app.editor.Image} imageModule
    @constructor
  ###
  constructor: (documentStore, editorStore, blockModule, sectionModule, imageModule) ->
    AppendButton = app.ui.AppendButton.component
    Section = sectionModule.reactClass
    Block = blockModule.reactClass
    ImageComponent = imageModule.reactClass
    {div, input, button} = React.DOM

    @reactClass = React.createClass(
      displayName: 'editor'

      getInitialState: ->
        return {
          doc: editorStore.getDocument()
        }

      render: ->
        div(
          {}
          div(
            {}
            @attributes()
            div(
              {}
              @renderBlocks(@state.doc.blocks)
            )
            app.BlockType.getAll().map @addBtn
          )
        )

      ###*
        @param {app.BlockType} type
      ###
      addBtn: (type) ->
        app.ui.AppendButton.component({
          type: type
          appendHandler: @onAppendBlock.bind(null, @state.doc, type)
        })

      attributes: ->
        input(
          {
            value: @state.doc.title
            onChange: @onDocumentTitleChange
            className: 'document-title'
            title: 'document title'
          }
        )

      shouldBeBlockActivated: (block) ->
        return (
          (block != @props.appState.activeBlock) &&
          (block._id == @props.appState.shouldActivateBlockId)
        )

      isBlockActive: (block) ->
        return (@props.appState.shouldActivateBlockId == block._id)

      ###*
        @param {Event} e
      ###
      onDocumentTitleChange: (e) ->
        newTitle = e.target.value
        editorStore.changeTitle @state.doc, newTitle

      componentDidMount: ->
        editorStore.listen app.EditorStore.EventType.Change, @onStoreChange

      componentWillUnmount: ->
        editorStore.unlisten app.EditorStore.EventType.Change, @onStoreChange

      ###*
        @param store {app.DocumentStore} - which store has updated
      ###
      onStoreChange: (store) ->
        if @isMounted()
          @forceUpdate()
        else
          console.warn 'editor store Not mounted on store change' if app.debug

      ###*
        @param block {app.document.StructuredBlock} - structured block where to append block
        @param type {app.BlockType} - type of block to append
      ###
      onAppendBlock: (block, type) ->
        console.error 'Must be called with block and type' unless block? && type?
        if app.BlockType.isStructure(type)
          editorStore.appendStructure block, type
        else
          editorStore.appendTextBlock block, type

      onBlockActivation: (block) ->
        editorStore.setAppState({activeBlock: block})

      onBlockRemove: (section) ->
        editorStore.deleteBlock section

      ###*
        @param block {app.document.Section}
        @param changeObject {app.document.Section}
      ###
      onSectionChange: (block, changeObject) ->
        editorStore.changeSection block, changeObject

      onTitleChange: (block, value) ->
        editorStore.changeTitle block, value

      onImageLoad: (imageBlock, file) ->
        reader = new FileReader()
        reader.readAsDataURL file
        reader.onload = (e) =>
          data = e.target.result
          editorStore.addImageData(imageBlock, data)


      ###*
        Maps structured block to according view.
      ###
      structuredBlockToView: (block) ->
        switch block.type
          when app.BlockType.Section
            canNest = documentStore.canNestStructured(block)
            sectionProps = {
              block: block
              onAppendBlock: @onAppendBlock
              onChange: @onSectionChange
              canNestSection: canNest
              onRemove: @onBlockRemove
              onBlockActivation: @onBlockActivation
              renderBlocks: @renderBlocks
              shouldActivate: @shouldBeBlockActivated(block)
              isActive: @isBlockActive(block)
            }
            Section sectionProps
          else
            console.warn('no block type for: ', block)


      textBlockToView: (block) ->
        switch block.type
          when app.BlockType.Image
            props = {
              block: block
              onRemove: @onBlockRemove
              onTitleChange: @onTitleChange
              onImageLoad: @onImageLoad
              showsImage: !!block.imageData
              # activation
              onBlockActivation: @onBlockActivation
              isActive: @isBlockActive(block)
              shouldActivate: @shouldBeBlockActivated(block)
            }
            ImageComponent props

          when app.BlockType.Paragraph, app.BlockType.List
            changeHandler = switch block.type
              when app.BlockType.Paragraph then editorStore.changeParagraphContent
              when app.BlockType.List then editorStore.changeListContent

            blockProps = {
              shouldActivate: @shouldBeBlockActivated(block)
              isActive: @isBlockActive(block)
              types: app.BlockType.getElementsInterchangeable()
              onEditorChange: changeHandler
              onBlockActivation: @onBlockActivation
              block: block
              renderBlocks: @renderBlocks
            }
            return Block blockProps

          else
            console.warn 'no block type'

      renderBlocks: (blocks) ->
        unless blocks?
          console.log('no blocks provided') if app.lamebug
          return null

        return blocks.map (block) =>
          if block instanceof app.document.StructuredBlock
            return @structuredBlockToView block
          else
            return @textBlockToView block








    )



