goog.provide 'app.editor.Image'

goog.require 'app.ui.ActivableBlock'
goog.require 'app.AbstractType'
goog.require 'app.ui.RemoveButton'
goog.require 'app.ui.AppendButton'
goog.require 'app.editor.Block'
goog.require 'app.ui.Message'

###
  Section can have subsections. Can be nested to 3 levels.
###
class app.editor.Image
  ###*
    @param {app.ui.ShortcutsMixin} shortcuts
    @constructor
  ###
  constructor: (shortcuts) ->
    {span, section, input, div, img} = React.DOM
    # http://www.sitepoint.com/web-foundations/mime-types-complete-list/
    acceptedMimes = [
      'image/png'
      'image/jpeg'
      'image/gif'
    ]

    @reactClass = React.createClass(

      mixins: [shortcuts.mixin, app.ui.ActivableBlock]

      getInitialState: ->
        return {
          isDropping: false
          inputError: undefined
        }

      getDefaultProps: ->
        return {
          showsImage: false
        }

  #    shouldComponentUpdate: (nextProps) ->
  #      if @props.showsImage
  #        return false
  #      return true

      render: ->
        active = if @props.isActive then 'block-active' else ''
        hoverState = if @state.isDropping then 'is-dropping' else ''
        return div(
          {
            className: "block image #{active}"
            key: @props.block._id
            title: @props.block.title
            onClick: @onClick
            onKeyDown: @onShortcut
          }
          if @state.inputError
            app.ui.Message({alert: @state.inputError, done: @resetInputError})
          @renderTitle()
          section(
            {className: "image-dropzone #{hoverState}"}
            @ImageDataSection()
          )
        )

      renderTitle: ->
        div(
          {}
          input(
            {
              ref: 'title'
              value: @props.block.title
              onChange: @onTitleChange
              className: 'image-title'
              placeholder: 'new image'
            }
          )
          app.ui.RemoveButton({title: 'remove image', onRemove: @onRemove, classSuffix: 'image'})
        )

      resetInputError: ->
        @refs.dropzone.getDOMNode().value = ''
        @setState({inputError: undefined, isDropping: false})

      onTitleChange: (e) ->
        @props.onTitleChange @props.block, e.target.value

      ImageDataSection: ->
        if @props.showsImage
          return img({src: @props.block.imageData})
        else
          return @DropZone()

      DropZone: ->
        return div(
          {
            onDragOver: @onDragover
            onDragLeave: @onDragleave
          }
          div(
            {className: 'drop-text'}
            'Drop image or click.'
          )
          input(
            {
              type: 'file', accept: 'image/*'
              ref: 'dropzone'
              className: 'drop-input'
              onChange: @onFileInput
            }
          )
        )

      onRemove: ->
        @props.onRemove @props.block

      onDragover: ->
        @setState {isDropping: true}

      onDragleave: ->
        @setState {isDropping: false}

      onFileInput: (e) ->
        fileInput = e.target
        file = fileInput.files[0]
        if file.type in acceptedMimes
          @props.onImageLoad @props.block, file
        else
          @setState {
            inputError: "This image type '#{file.type}' is not allowed. You can use jpeg/png/gif."
          }
    )

