goog.provide 'app.formatter.Store'

goog.require 'goog.events.EventTarget'
goog.require 'app.formatter.IFormatterStorage'

class app.formatter.Store extends goog.events.EventTarget

  ###*
    @enum {string}
  ###
  @EventType: {
    Change: 'formatter-change'
  }

  ###*
    @constructor
    @extends {goog.events.EventTarget}
    @param {app.EditorStore} editorStore
    @final
  ###
  constructor: (@editorStore) ->
    super()

  ###*
    Saves current state of formatter. Saves to localStorage and dispatch event for
    TOP level component {app.formatter.Document}.
    @private
  ###
  _notify: ->
    @dispatchEvent app.formatter.Store.EventType.Change
    @_saveFontSettings()

  _saveFontSettings: ->
    @editorStore.forceDocumentSave()
    # TODO: improve

  ###*
    @return {app.formatter.DocumentSettings}
  ###
  getSettings: ->
    return @editorStore.doc.formatSettings

  changeFontFamily: (fontKey) ->
    @editorStore.doc.formatSettings.fontFamily = fontKey
    @_notify()

  changeTextSize: (newSize) ->
    @editorStore.doc.formatSettings.fontSize = newSize
    @_notify()

  ###*
    @param {app.document.Image} image
    @param {app.formatter.ImageSettings.Format} format
  ###
  changeImageFormat: (image, format) ->
    image.formatSettings.format = format
    @_notify()


